@@ -45,7 +45,6 @@ import xarray
 from aesara import tensor as at
 from aesara.graph.basic import Apply, Constant, Variable, general_toposort, walk
 from aesara.graph.fg import FunctionGraph
-from aesara.tensor.random.op import RandomVariable
 from aesara.tensor.random.var import (
     RandomGeneratorSharedVariable,
     RandomStateSharedVariable,
@@ -71,6 +70,7 @@ from pymc.initial_point import (
 )
 from pymc.model import Model, modelcontext
 from pymc.parallel_sampling import Draw, _cpu_count
+from pymc.stats.convergence import run_convergence_checks
 from pymc.step_methods import NUTS, CompoundStep, DEMetropolis
 from pymc.step_methods.arraystep import BlockedStep, PopulationArrayStepShared
 from pymc.step_methods.hmc import quadpotential
@@ -312,7 +312,6 @@ def sample(
     n_init: int = 200_000,
     initvals: Optional[Union[StartDict, Sequence[Optional[StartDict]]]] = None,
     trace: Optional[Union[BaseTrace, List[str]]] = None,
-    chain_idx: int = 0,
     chains: Optional[int] = None,
     cores: Optional[int] = None,
     tune: int = 1000,
@@ -354,9 +353,6 @@ def sample(
     trace : backend or list
         This should be a backend instance, or a list of variables to track.
         If None or a list of variables, the NDArray backend is used.
-    chain_idx : int
-        Chain number used to store sample in backend. If ``chains`` is greater than one, chain
-        numbers will start here.
     chains : int
         The number of chains to sample. Running independent chains is important for some
         convergence statistics and can also reveal multiple modes in the posterior. If ``None``,
@@ -570,7 +566,6 @@ def sample(
         "step": step,
         "start": initial_points,
         "trace": trace,
-        "chain": chain_idx,
         "chains": chains,
         "tune": tune,
         "progressbar": progressbar,
@@ -659,7 +654,7 @@ def sample(
     # count the number of tune/draw iterations that happened
     # ideally via the "tune" statistic, but not all samplers record it!
     if "tune" in mtrace.stat_names:
-        stat = mtrace.get_sampler_stats("tune", chains=chain_idx)
+        stat = mtrace.get_sampler_stats("tune", chains=0)
         # when CompoundStep is used, the stat is 2 dimensional!
         if len(stat.shape) == 2:
             stat = stat[:, 0]
@@ -683,7 +678,7 @@ def sample(
     _log.info(
         f'Sampling {n_chains} chain{"s" if n_chains > 1 else ""} for {n_tune:_d} tune and {n_draws:_d} draw iterations '
         f"({n_tune*n_chains:_d} + {n_draws*n_chains:_d} draws total) "
-        f"took {mtrace.report.t_sampling:.0f} seconds."
+        f"took {t_sampling:.0f} seconds."
     )
     mtrace.report._log_summary()
 
@@ -701,7 +696,8 @@ def sample(
                     stacklevel=2,
                 )
             else:
-                mtrace.report._run_convergence_checks(idata, model)
+                convergence_warnings = run_convergence_checks(idata, model)
+                mtrace.report._add_warnings(convergence_warnings)
 
         if return_inferencedata:
             return idata
@@ -735,7 +731,6 @@ def _check_start_shape(model, start: PointType):
 
 def _sample_many(
     draws: int,
-    chain: int,
     chains: int,
     start: Sequence[PointType],
     random_seed: Optional[Sequence[RandomSeed]],
@@ -749,8 +744,6 @@ def _sample_many(
     ----------
     draws: int
         The number of samples to draw
-    chain: int
-        Number of the first chain in the sequence.
     chains: int
         Total number of chains to sample.
     start: list
@@ -769,7 +762,7 @@ def _sample_many(
     for i in range(chains):
         trace = _sample(
             draws=draws,
-            chain=chain + i,
+            chain=i,
             start=start[i],
             step=step,
             random_seed=None if random_seed is None else random_seed[i],
@@ -792,7 +785,6 @@ def _sample_many(
 
 def _sample_population(
     draws: int,
-    chain: int,
     chains: int,
     start: Sequence[PointType],
     random_seed: RandomSeed,
@@ -809,8 +801,6 @@ def _sample_population(
     ----------
     draws : int
         The number of samples to draw
-    chain : int
-        The number of the first chain in the population
     chains : int
         The total number of chains in the population
     start : list
@@ -833,7 +823,6 @@ def _sample_population(
     """
     sampling = _prepare_iter_population(
         draws,
-        [chain + c for c in range(chains)],
         step,
         start,
         parallelize,
@@ -953,8 +942,7 @@ def iter_sample(
         This should be a backend instance, or a list of variables to track.
         If None or a list of variables, the NDArray backend is used.
     chain : int, optional
-        Chain number used to store sample in backend. If ``cores`` is greater than one, chain numbers
-        will start here.
+        Chain number used to store sample in backend.
     tune : int, optional
         Number of iterations to tune (defaults to 0).
     model : Model (optional if in ``with`` context)
@@ -1009,8 +997,7 @@ def _iter_sample(
         This should be a backend instance, or a list of variables to track.
         If None or a list of variables, the NDArray backend is used.
     chain : int, optional
-        Chain number used to store sample in backend. If ``cores`` is greater than one, chain numbers
-        will start here.
+        Chain number used to store sample in backend.
     tune : int, optional
         Number of iterations to tune (defaults to 0).
     model : Model (optional if in ``with`` context)
@@ -1248,7 +1235,6 @@ class PopulationStepper:
 
 def _prepare_iter_population(
     draws: int,
-    chains: list,
     step,
     start: Sequence[PointType],
     parallelize: bool,
@@ -1263,8 +1249,6 @@ def _prepare_iter_population(
     ----------
     draws : int
         The number of samples to draw
-    chains : list
-        The chain numbers in the population
     step : function
         Step function (should be or contain a population step method)
     start : list
@@ -1283,8 +1267,7 @@ def _prepare_iter_population(
     _iter_population : generator
         Yields traces of all chains at the same time
     """
-    # chains contains the chain numbers, but for indexing we need indices...
-    nchains = len(chains)
+    nchains = len(start)
     model = modelcontext(model)
     draws = int(draws)
 
@@ -1328,7 +1311,7 @@ def _prepare_iter_population(
             trace=None,
             model=model,
         )
-        for c in chains
+        for c in range(nchains)
     ]
 
     # 4. configure the PopulationStepper (expensive call)
@@ -1458,7 +1441,6 @@ def _mp_sample(
     step,
     chains: int,
     cores: int,
-    chain: int,
     random_seed: Sequence[RandomSeed],
     start: Sequence[PointType],
     progressbar: bool = True,
@@ -1483,8 +1465,6 @@ def _mp_sample(
         The number of chains to sample.
     cores : int
         The number of chains to run in parallel.
-    chain : int
-        Number of the first chain.
     random_seed : list of random seeds
         Random seeds for each chain.
     start : list
@@ -1521,26 +1501,25 @@ def _mp_sample(
             trace=trace,
             model=model,
         )
-        for chain_number in range(chain, chain + chains)
+        for chain_number in range(chains)
     ]
 
     sampler = ps.ParallelSampler(
-        draws,
-        tune,
-        chains,
-        cores,
-        random_seed,
-        start,
-        step,
-        chain,
-        progressbar,
+        draws=draws,
+        tune=tune,
+        chains=chains,
+        cores=cores,
+        seeds=random_seed,
+        start_points=start,
+        step_method=step,
+        progressbar=progressbar,
         mp_ctx=mp_ctx,
     )
     try:
         try:
             with sampler:
                 for draw in sampler:
-                    strace = traces[draw.chain - chain]
+                    strace = traces[draw.chain]
                     if strace.supports_sampler_stats and draw.stats is not None:
                         strace.record(draw.point, draw.stats)
                     else:
@@ -1554,7 +1533,7 @@ def _mp_sample(
                         callback(trace=trace, draw=draw)
 
         except ps.ParallelSamplingError as error:
-            strace = traces[error._chain - chain]
+            strace = traces[error._chain]
             strace._add_warnings(error._warnings)
             for strace in traces:
                 strace.close()
@@ -1622,8 +1601,10 @@ def compile_forward_sampling_function(
     vars_in_trace: List[Variable],
     basic_rvs: Optional[List[Variable]] = None,
     givens_dict: Optional[Dict[Variable, Any]] = None,
+    constant_data: Optional[Dict[str, np.ndarray]] = None,
+    constant_coords: Optional[Set[str]] = None,
     **kwargs,
-) -> Callable[..., Union[np.ndarray, List[np.ndarray]]]:
+) -> Tuple[Callable[..., Union[np.ndarray, List[np.ndarray]]], Set[Variable]]:
     """Compile a function to draw samples, conditioned on the values of some variables.
 
     The goal of this function is to walk the aesara computational graph from the list
@@ -1635,21 +1616,18 @@ def compile_forward_sampling_function(
     compiled function or after inference has been run. These variables are:
 
     - Variables in the outputs list
-    - ``SharedVariable`` instances that are not ``RandomStateSharedVariable`` or ``RandomGeneratorSharedVariable``
-    - Basic RVs that are not in the ``vars_in_trace`` list
+    - ``SharedVariable`` instances that are not ``RandomStateSharedVariable`` or ``RandomGeneratorSharedVariable``, and whose values changed with respect to what they were at inference time
+    - Variables that are in the `basic_rvs` list but not in the ``vars_in_trace`` list
     - Variables that are keys in the ``givens_dict``
     - Variables that have volatile inputs
 
-    Where by basic RVs we mean ``Variable`` instances produced by a ``RandomVariable`` ``Op``
-    that are in the ``basic_rvs`` list.
-
     Concretely, this function can be used to compile a function to sample from the
     posterior predictive distribution of a model that has variables that are conditioned
-    on ``MutableData`` instances. The variables that depend on the mutable data will be
-    considered volatile, and as such, they wont be included as inputs into the compiled function.
-    This means that if they have values stored in the posterior, these values will be ignored
-    and new values will be computed (in the case of deterministics and potentials) or sampled
-    (in the case of random variables).
+    on ``MutableData`` instances. The variables that depend on the mutable data that have changed
+    will be considered volatile, and as such, they wont be included as inputs into the compiled
+    function. This means that if they have values stored in the posterior, these values will be
+    ignored and new values will be computed (in the case of deterministics and potentials) or
+    sampled (in the case of random variables).
 
     This function also enables a way to impute values for any variable in the computational
     graph that produces the desired outputs: the ``givens_dict``. This dictionary can be used
@@ -1671,12 +1649,38 @@ def compile_forward_sampling_function(
         output of ``model.basic_RVs``) should have a reference to the variables that should
         be considered as random variable instances. This includes variables that have
         a ``RandomVariable`` owner op, but also unpure random variables like Mixtures, or
-        Censored distributions. If ``None``, only pure random variables will be considered
-        as potential random variables.
+        Censored distributions.
     givens_dict : Optional[Dict[aesara.graph.basic.Variable, Any]]
         A dictionary that maps tensor variables to the values that should be used to replace them
         in the compiled function. The types of the key and value should match or an error will be
         raised during compilation.
+    constant_data : Optional[Dict[str, numpy.ndarray]]
+        A dictionary that maps the names of ``MutableData`` or ``ConstantData`` instances to their
+        corresponding values at inference time. If a model was created with ``MutableData``, these
+        are stored as ``SharedVariable`` with the name of the data variable and a value equal to
+        the initial data. At inference time, this information is stored in ``InferenceData``
+        objects under the ``constant_data`` group, which allows us to check whether a
+        ``SharedVariable`` instance changed its values after inference or not. If the values have
+        changed, then the ``SharedVariable`` is assumed to be volatile. If it has not changed, then
+        the ``SharedVariable`` is assumed to not be volatile. If a ``SharedVariable`` is not found
+        in either ``constant_data`` or ``constant_coords``, then it is assumed to be volatile.
+        Setting ``constant_data`` to ``None`` is equivalent to passing an empty dictionary.
+    constant_coords : Optional[Set[str]]
+        A set with the names of the mutable coordinates that have not changed their shape after
+        inference. If a model was created with mutable coordinates, these are stored as
+        ``SharedVariable`` with the name of the coordinate and a value equal to the length of said
+        coordinate. This set let's us check if a ``SharedVariable`` is a mutated coordinate, in
+        which case, it is considered volatile. If a ``SharedVariable`` is not found
+        in either ``constant_data`` or ``constant_coords``, then it is assumed to be volatile.
+        Setting ``constant_coords`` to ``None`` is equivalent to passing an empty set.
+
+    Returns
+    -------
+    function: Callable
+        Compiled forward sampling Aesara function
+    volatile_basic_rvs: Set of Variable
+        Set of all basic_rvs that were considered volatile and will be resampled when
+        the function is evaluated
     """
     if givens_dict is None:
         givens_dict = {}
@@ -1684,6 +1688,20 @@ def compile_forward_sampling_function(
     if basic_rvs is None:
         basic_rvs = []
 
+    if constant_data is None:
+        constant_data = {}
+    if constant_coords is None:
+        constant_coords = set()
+
+    # We define a helper function to check if shared values match to an array
+    def shared_value_matches(var):
+        try:
+            old_array_value = constant_data[var.name]
+        except KeyError:
+            return var.name in constant_coords
+        current_shared_value = var.get_value(borrow=True)
+        return np.array_equal(old_array_value, current_shared_value)
+
     # We need a function graph to walk the clients and propagate the volatile property
     fg = FunctionGraph(outputs=outputs, clone=False)
 
@@ -1699,12 +1717,10 @@ def compile_forward_sampling_function(
             or (  # SharedVariables, except RandomState/Generators
                 isinstance(node, SharedVariable)
                 and not isinstance(node, (RandomStateSharedVariable, RandomGeneratorSharedVariable))
+                and not shared_value_matches(node)
             )
             or (  # Basic RVs that are not in the trace
-                node.owner
-                and isinstance(node.owner.op, RandomVariable)
-                and node in basic_rvs
-                and node not in vars_in_trace
+                node in basic_rvs and node not in vars_in_trace
             )
             or (  # Variables that have any volatile input
                 node.owner and any(inp in volatile_nodes for inp in node.owner.inputs)
@@ -1745,7 +1761,10 @@ def compile_forward_sampling_function(
         for node, value in givens_dict.items()
     ]
 
-    return compile_pymc(inputs, fg.outputs, givens=givens, on_unused_input="ignore", **kwargs)
+    return (
+        compile_pymc(inputs, fg.outputs, givens=givens, on_unused_input="ignore", **kwargs),
+        set(basic_rvs) & (volatile_nodes - set(givens_dict)),  # Basic RVs that will be resampled
+    )
 
 
 def sample_posterior_predictive(
@@ -1832,16 +1851,24 @@ def sample_posterior_predictive(
         idata_kwargs = {}
     else:
         idata_kwargs = idata_kwargs.copy()
+    constant_data: Dict[str, np.ndarray] = {}
+    trace_coords: Dict[str, np.ndarray] = {}
     if "coords" not in idata_kwargs:
         idata_kwargs["coords"] = {}
     if isinstance(trace, InferenceData):
         idata_kwargs["coords"].setdefault("draw", trace["posterior"]["draw"])
         idata_kwargs["coords"].setdefault("chain", trace["posterior"]["chain"])
+        _constant_data = getattr(trace, "constant_data", None)
+        if _constant_data is not None:
+            trace_coords.update({str(k): v.data for k, v in _constant_data.coords.items()})
+            constant_data.update({str(k): v.data for k, v in _constant_data.items()})
+        trace_coords.update({str(k): v.data for k, v in trace["posterior"].coords.items()})
         _trace = dataset_to_point_list(trace["posterior"])
         nchain, len_trace = chains_and_samples(trace)
     elif isinstance(trace, xarray.Dataset):
         idata_kwargs["coords"].setdefault("draw", trace["draw"])
         idata_kwargs["coords"].setdefault("chain", trace["chain"])
+        trace_coords.update({str(k): v.data for k, v in trace.coords.items()})
         _trace = dataset_to_point_list(trace)
         nchain, len_trace = chains_and_samples(trace)
     elif isinstance(trace, MultiTrace):
@@ -1898,13 +1925,22 @@ def sample_posterior_predictive(
             stacklevel=2,
         )
 
+    constant_coords = set()
+    for dim, coord in trace_coords.items():
+        current_coord = model.coords.get(dim, None)
+        if (
+            current_coord is not None
+            and len(coord) == len(current_coord)
+            and np.all(coord == current_coord)
+        ):
+            constant_coords.add(dim)
+
     if var_names is not None:
         vars_ = [model[x] for x in var_names]
     else:
         vars_ = model.observed_RVs + model.auto_deterministics
 
     indices = np.arange(samples)
-
     if progressbar:
         indices = progress_bar(indices, total=samples, display=progressbar)
 
@@ -1927,31 +1963,27 @@ def sample_posterior_predictive(
     compile_kwargs.setdefault("allow_input_downcast", True)
     compile_kwargs.setdefault("accept_inplace", True)
 
-    sampler_fn = point_wrapper(
-        compile_forward_sampling_function(
-            outputs=vars_to_sample,
-            vars_in_trace=vars_in_trace,
-            basic_rvs=model.basic_RVs,
-            givens_dict=None,
-            random_seed=random_seed,
-            **compile_kwargs,
-        )
+    _sampler_fn, volatile_basic_rvs = compile_forward_sampling_function(
+        outputs=vars_to_sample,
+        vars_in_trace=vars_in_trace,
+        basic_rvs=model.basic_RVs,
+        givens_dict=None,
+        random_seed=random_seed,
+        constant_data=constant_data,
+        constant_coords=constant_coords,
+        **compile_kwargs,
     )
-
+    sampler_fn = point_wrapper(_sampler_fn)
+    # All model variables have a name, but mypy does not know this
+    _log.info(f"Sampling: {list(sorted(volatile_basic_rvs, key=lambda var: var.name))}")  # type: ignore
     ppc_trace_t = _DefaultTrace(samples)
     try:
-        if isinstance(_trace, MultiTrace):
-            # trace dict is unordered, but we want to return ppc samples in
-            # a predictable ordering, so sort the chain indices
-            chain_idx_mapping = sorted(_trace._straces.keys())
         for idx in indices:
             if nchain > 1:
                 # the trace object will either be a MultiTrace (and have _straces)...
                 if hasattr(_trace, "_straces"):
                     chain_idx, point_idx = np.divmod(idx, len_trace)
                     chain_idx = chain_idx % nchain
-                    # chain indices might not always start at 0, convert to proper index
-                    chain_idx = chain_idx_mapping[chain_idx]
                     param = cast(MultiTrace, _trace)._straces[chain_idx].point(point_idx)
                 # ... or a PointList
                 else:
@@ -2246,7 +2278,7 @@ def sample_prior_predictive(
     compile_kwargs.setdefault("allow_input_downcast", True)
     compile_kwargs.setdefault("accept_inplace", True)
 
-    sampler_fn = compile_forward_sampling_function(
+    sampler_fn, volatile_basic_rvs = compile_forward_sampling_function(
         vars_to_sample,
         vars_in_trace=[],
         basic_rvs=model.basic_RVs,
@@ -2255,6 +2287,8 @@ def sample_prior_predictive(
         **compile_kwargs,
     )
 
+    # All model variables have a name, but mypy does not know this
+    _log.info(f"Sampling: {list(sorted(volatile_basic_rvs, key=lambda var: var.name))}")  # type: ignore
     values = zip(*(sampler_fn() for i in range(samples)))
 
     data = {k: np.stack(v) for k, v in zip(names, values)}
