@@ -18,9 +18,10 @@ import aesara
 import numpy as np
 import pytest
 
-from aesara import Mode
 from aesara import tensor as at
+from aesara.compile.mode import Mode
 from aesara.graph import Constant, ancestors
+from aesara.tensor import TensorVariable
 from aesara.tensor.random import normal
 from aesara.tensor.shape import SpecifyShape
 
@@ -36,10 +37,13 @@ from pymc.distributions.shape_utils import (
     convert_shape,
     convert_size,
     get_broadcastable_dist_samples,
+    get_support_shape,
+    get_support_shape_1d,
     rv_size_is_none,
     shapes_broadcasting,
     to_tuple,
 )
+from pymc.model import Model
 
 test_shapes = [
     (tuple(), (1,), (4,), (5, 4)),
@@ -214,7 +218,7 @@ class TestSamplesBroadcasting:
                 broadcast_dist_samples_to(to_shape, samples, size=size)
 
 
-class TestShapeDimsSize:
+class TestSizeShapeDimsObserved:
     @pytest.mark.parametrize("param_shape", [(), (2,)])
     @pytest.mark.parametrize("batch_shape", [(), (3,)])
     @pytest.mark.parametrize(
@@ -461,6 +465,13 @@ class TestShapeDimsSize:
         # draw, would match the first value of the second draw
         assert fn()[1] != fn()[0]
 
+    def test_explicit_size_shape_none(self):
+        with pm.Model() as m:
+            x = pm.Normal("x", shape=None, observed=[1, 2, 3])
+            y = pm.Normal("y", size=None, observed=[1, 2, 3, 4])
+        assert x.shape.eval().item() == 3
+        assert y.shape.eval().item() == 4
+
 
 def test_rv_size_is_none():
     rv = pm.Normal.dist(0, 1, size=None)
@@ -599,3 +610,141 @@ def test_change_specify_shape_size_multivariate():
         new_x.eval({batch: 5, supp: 3}).shape == (10, 5, 5, 3)
         with pytest.raises(AssertionError, match=re.escape("expected (None, None, 5, 3)")):
             new_x.eval({batch: 6, supp: 3}).shape == (10, 5, 5, 3)
+
+
+@pytest.mark.parametrize(
+    "support_shape, shape, support_shape_offset, expected_support_shape, consistent",
+    [
+        (10, None, 0, 10, True),
+        (10, None, 1, 10, True),
+        (None, (10,), 0, 10, True),
+        (None, (10,), 1, 9, True),
+        (None, (10, 5), 0, 5, True),
+        (None, None, 0, None, True),
+        (10, (10,), 0, 10, True),
+        (10, (11,), 1, 10, True),
+        (10, (5, 5), 0, 5, False),
+        (10, (5, 10), 1, 9, False),
+    ],
+)
+@pytest.mark.parametrize("info_source", ("shape", "dims", "observed"))
+def test_get_support_shape_1d(
+    info_source, support_shape, shape, support_shape_offset, expected_support_shape, consistent
+):
+    if info_source == "shape":
+        inferred_support_shape = get_support_shape_1d(
+            support_shape=support_shape, shape=shape, support_shape_offset=support_shape_offset
+        )
+
+    elif info_source == "dims":
+        if shape is None:
+            dims = None
+            coords = {}
+        else:
+            dims = tuple(str(i) for i, _ in enumerate(shape))
+            coords = {str(i): range(shape) for i, shape in enumerate(shape)}
+        with Model(coords=coords):
+            inferred_support_shape = get_support_shape_1d(
+                support_shape=support_shape, dims=dims, support_shape_offset=support_shape_offset
+            )
+
+    elif info_source == "observed":
+        if shape is None:
+            observed = None
+        else:
+            observed = np.zeros(shape)
+        inferred_support_shape = get_support_shape_1d(
+            support_shape=support_shape,
+            observed=observed,
+            support_shape_offset=support_shape_offset,
+        )
+
+    if not isinstance(inferred_support_shape, TensorVariable):
+        assert inferred_support_shape == expected_support_shape
+    else:
+        if consistent:
+            assert inferred_support_shape.eval() == expected_support_shape
+        else:
+            # check that inferred steps is still correct by ignoring the assert
+            f = aesara.function(
+                [], inferred_support_shape, mode=Mode().including("local_remove_all_assert")
+            )
+            assert f() == expected_support_shape
+            with pytest.raises(AssertionError, match="support_shape does not match"):
+                inferred_support_shape.eval()
+
+
+@pytest.mark.parametrize(
+    "support_shape, shape, support_shape_offset, expected_support_shape, ndim_supp, consistent",
+    [
+        ((10, 5), None, (0,), (10, 5), 1, True),
+        ((10, 5), None, (1, 1), (10, 5), 1, True),
+        (None, (10, 5), (0,), 5, 1, True),
+        (None, (10, 5), (1,), 4, 1, True),
+        (None, (10, 5, 2), (0,), 2, 1, True),
+        (None, None, None, None, 1, True),
+        ((10, 5), (10, 5), None, (10, 5), 2, True),
+        ((10, 5), (11, 10, 5), None, (10, 5), 2, True),
+        (None, (11, 10, 5), (0, 1, 0), (11, 9, 5), 3, True),
+        ((10, 5), (10, 5, 5), (0,), (5,), 1, False),
+        ((10, 5), (10, 5), (1, 1), (9, 4), 2, False),
+    ],
+)
+@pytest.mark.parametrize("info_source", ("shape", "dims", "observed"))
+def test_get_support_shape(
+    info_source,
+    support_shape,
+    shape,
+    support_shape_offset,
+    expected_support_shape,
+    ndim_supp,
+    consistent,
+):
+    if info_source == "shape":
+        inferred_support_shape = get_support_shape(
+            support_shape=support_shape,
+            shape=shape,
+            support_shape_offset=support_shape_offset,
+            ndim_supp=ndim_supp,
+        )
+
+    elif info_source == "dims":
+        if shape is None:
+            dims = None
+            coords = {}
+        else:
+            dims = tuple(str(i) for i, _ in enumerate(shape))
+            coords = {str(i): range(shape) for i, shape in enumerate(shape)}
+        with Model(coords=coords):
+            inferred_support_shape = get_support_shape(
+                support_shape=support_shape,
+                dims=dims,
+                support_shape_offset=support_shape_offset,
+                ndim_supp=ndim_supp,
+            )
+
+    elif info_source == "observed":
+        if shape is None:
+            observed = None
+        else:
+            observed = np.zeros(shape)
+        inferred_support_shape = get_support_shape(
+            support_shape=support_shape,
+            observed=observed,
+            support_shape_offset=support_shape_offset,
+            ndim_supp=ndim_supp,
+        )
+
+    if not isinstance(inferred_support_shape, TensorVariable):
+        assert inferred_support_shape == expected_support_shape
+    else:
+        if consistent:
+            assert (inferred_support_shape.eval() == expected_support_shape).all()
+        else:
+            # check that inferred support shape is still correct by ignoring the assert
+            f = aesara.function(
+                [], inferred_support_shape, mode=Mode().including("local_remove_all_assert")
+            )
+            assert (f() == expected_support_shape).all()
+            with pytest.raises(AssertionError, match="support_shape does not match"):
+                inferred_support_shape.eval()
